use gtk;
import Container;
import Label;
import Image;

/**
 * A button can be clicked
 */
class Button extends Container {

	GtkButton* button;
	
	implement getObject {
		return GTK_OBJECT(button);
	}

	/**
	 * Create a new, empty, button.
	 */
	new() {
		button = GTK_BUTTON(gtk_button_new());
	}
	
	/**
	 * Create a new button from stock.
	 * @param stockId the stock identifier, e.g. "gtk-media-play" or "gtk-ok"
	 */
	new(String stockId) {
		button = GTK_BUTTON(gtk_button_new_from_stock(stockId));
	}
	
	void setLabel(String text) {
		gtk_button_set_label(button, text);
	}
	
	void setUseStock(gboolean useStock) {
		gtk_button_set_use_stock(button, useStock);
	}
	
	void setImage(Image image) {
		gtk_button_set_image(button, GTK_IMAGE(image.getObject()));
	}
	
	void pressed() {
		gtk_button_pressed(button);
	}
	
	static Button newTextButton(String text) {
		
		Button button = new Button();
		button.setLabel(text);
		return button;
		
	}
	
	static Button newTextButton(String text, Function callback) {
		
		Button button = new Button();
		button.setLabel(text);
		button.connect("clicked", callback);
		return button;
		
	}
	
	static Button newFromStock(String stockId, Function callback) {
		
		Button button = new Button(stockId);
		button.connect("clicked", callback);
		return button;
		
	}
	
	static Button newFromStock(String stockId, String text, Function callback) {
		
		Button button = new Button();
		button.setLabel(text);
		button.setImage(new Image(stockId, GTK_ICON_SIZE_LARGE_TOOLBAR));
		button.connect("clicked", callback);
		return button;
		
	}

}
