use gtk;

class SignalPacket {
	
	GObject object;
	gpointer data;
	Function callback;
	
	new(=object, =data, =callback) {
		//printf("new SignalPacket created with a %s and data '%s'\n", object.class->name, (String) data);
	}
	
	void call() {
		//printf("[SignalPacket] Forwarding callback on a %s with data '%s'!\n", object.class->name, (String) data);
		callback(object, data);
	}
	
}

void signalWrapper(GtkObject* object, gpointer data) {
	
	//printf("[SignalPacket] signalWrapper, got called!\n");
	SignalPacket packet = (SignalPacket) data;
	packet.call();
	
}

/**
 * The base of the GObject hierarchy. Can send and receive signals
 */
abstract class GObject {

	/**
	 * must be implemented by any child class
	 */
	abstract GtkObject* getObject();

	/**
	 * Connect a callback to a signal
	 * @param signalName the name of the signal to connect to
	 * @param callback a pointer to the function to call whenever
	 * this signal is received
	 */
	void connect(String signalName, Function callback) {
		connect(signalName, callback, null);
	}

	/**
	 * Connect a callback to a signal
	 * @param signalName the name of the signal to connect to
	 * @param callback a pointer to the function to call whenever
	 * this signal is received
	 * @param data data to be passed as an argument to the callback
	 */
	void connect(String signalName, Function callback, gpointer data) {
		gulong id = gtk_signal_connect(GTK_OBJECT(this.getObject()), signalName, GTK_SIGNAL_FUNC(signalWrapper), new SignalPacket(this, data, callback));
		//printf("Just connected a %s to signal %s, and got callback ID %ld\n", this.class->name, signalName, id);
	}
	
	void connectNaked(String signalName, Function callback) {
		gulong id = gtk_signal_connect(GTK_OBJECT(this.getObject()), signalName, GTK_SIGNAL_FUNC(callback), null);
		//printf("Just naked-connected a %s to signal %s, and got callback ID %ld\n", this.class->name, signalName, id);
	}
	
	void emitByName(String signalName) {
		gtk_signal_emit_by_name(GTK_OBJECT(this.getObject()), signalName);
	}
	
	/**
	 * Add a reference to this object, thus preventing its destruction
	 */
	void ref() {
		gtk_object_ref(getObject());
	}
	
	/**
	 * Release a reference from this object
	 */
	void unref() {
		gtk_object_unref(getObject());
	}

	/**
	 * 'Take ownership' of this object, ie. remove its initial reference
	 */
	void sink() {
		gtk_object_sink(getObject());
	}
	
	/**
	 * Sets a property on an object.
	 * @param propertyName the name of the property to set
	 * @param value the value 
	 */
	void setProperty(String name, GValue* value) {
		g_object_set_property(this.getObject(), name, value);
	}

}
