use gtk;
import GObject;
import Style;

/**
 * A GTK widget, such as a Button, a Label, a Checkbox
 */
abstract class Widget extends GObject {

	GtkWidget* getWidget() {
		return GTK_WIDGET(this.getObject());
	}
	
	/**
	 * Set the sensitivity of this widget
	 * @param sensitive if true, the widget will react to the user
	 * input, and send/receive signals as usual. If false, thewidget
	 * will be "grayed out" and won't react to anything
	 */
	void setSensitive(boolean sensitive) {
		gtk_widget_set_sensitive(getWidget(), sensitive);
	}
	
	/**
	 * Realize this component on-screen, e.g. allocate resources, etc.
	 * It's often not needed to call it directly, use show() instead.
	 */
	void realize() {
		gtk_widget_realize(getWidget());
	}
	
	/**
	 * Shows this widget on-screen.
	 */
	void show() {
		gtk_widget_show(getWidget());
	}
	
	/**
	 * Shows this widget on-screen and all its children.
	 */
	void showAll() {
		gtk_widget_show_all(getWidget());
	}
	
	/**
	 * Hides this widget
	 */
	void hide() {
		gtk_widget_hide(getWidget());
	}
	
	void destroy() {
		gtk_widget_destroy(getWidget());
	}
	
	/**
	 * set the position of this wdiget
	 * @param x the x coordinate of the desired position for this widget, or
	 * -1 for default position
	 * @param y the y coordinate of the desired position for this widget, or
	 * -1 for default position
	 */
	void setPosition(gint x, gint y) {
		gtk_widget_set_usize(getWidget(), x, y);
	}
	
	/**
	 * set the size of this widget
	 * @param width the desired width for this widget, or -1 for the default
	 * @param height the desired height for this widget, or -1 for the default
	 */
	void setUSize(gint width, gint height) {
		gtk_widget_set_usize(getWidget(), width, height);
	}
	
	/**
	 * Adjust the event mask of this widget
	 */
	void setEvents(int eventMask) {
		gtk_widget_set_events(getWidget(), eventMask);
	}
	
	/**
	 * Return the size of the window, as a GtkAllocation (a GdkRectangle, really)
	 */
	GtkAllocation getAllocation() {
		return getWidget()->allocation;
	}
	
	/**
	 * The height of this window
	 */
	int getWidth() {
		return getWidget()->allocation.width;
	}
	
	/**
	 * The height of this window
	 */
	int getHeight() {
		return getWidget()->allocation.height;
	}
	
	Style getStyle() {
		return new Style(gtk_widget_get_style(getWidget()));
	}

}
