/**
 * List interface for a data container
 */
abstract class List {
	
	/**
	 * Appends the specified element to the end of this list.
	 */
	abstract void add(Object element);

	/**
	 * Inserts the specified element at the specified position in
	 * this list. 
	 */
	abstract void add(int index, Object element);
	
	/**
	 * Appends all of the elements in the specified Collection to the
	 * end of this list, in the order that they are returned by the
	 * specified Collection's Iterator.
	 */
	void addAll(List list) {
		addAll(0, list);
	}
	
	/**
	 * Inserts all of the elements in the specified Collection into
	 * this list, starting at the specified position.
	 */
	void addAll(int index, List list) {
		for(int i: 0..list.size()) {
			add(list.get(i));
		}
	}
	
	/**
	 * Removes all of the elements from this list.
	 */
	abstract void clear();
	
	/**
	 * Removes the last element of the list, if any (=non-empty list).
	 * @return true if it has removed an element, false if the list
	 * was empty.
	 */
	boolean removeLast() {
		int size = size();
		if(size > 0) {
			remove(size - 1);
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * @return true if this list contains the specified element.
	 */
	boolean contains(Object element) {
		return (indexOf(element) != -1);
	}
	
	/**
	 * @return the element at the specified position in this list.
	 */
	abstract Object get(int i);
	
	/**
	 * @return the index of the first occurence of the given argument,
	 * (testing for equality using the equals method), or -1 if not found
	 */
	abstract int indexOf(Object element);
	
	/**
	 * @return true if this list has no elements.
	 */
	boolean isEmpty() {
		return (size() == 0);
	}
	
	/**
	 * @return the index of the last occurrence of the specified object
	 * in this list.
	 */
	abstract int lastIndexOf(Object element);
	
	/**
	 * Removes the element at the specified position in this list.
	 * @return the element just removed
	 */
	abstract Object remove(int i);
	
	/**
	 * Removes a single instance of the specified element from this list,
	 * if it is present (optional operation).
	 * @return true if at least one occurence of the element has been
	 * removed
	 */
	abstract boolean removeElement(Object element);

	/**
	 * Replaces the element at the specified position in this list with
	 * the specified element.
	 */ 
	abstract void set(int i, Object element);
	
	/**
	 * @return the number of elements in this list.
	 */
	abstract int size();

	/**
	 * @return the number of elements this list can hold without needing
	 * to grow.
	 */
	abstract int capacity();

}
