include stdlib, stdio, memory;
import structs.List;

/**
 * A sparse list stores its data into an array, resizes it when it's too
 * small,
 * 
 */
class SparseList extends List {

	Object* data;
	int capacity;
	int size;
	
	new() {
		this(10);
	}

	new(=capacity) {
		size = 0;
		data = malloc(capacity * sizeof(Object));
		data[0] = null; // To null out the first element is capital!
	}
	
	=size size();
	=capacity capacity();
	
	void add(Object element) {
		int slot = getFreeSlot(-1);
		//printf("Adding to SparseList, capacity=%d, size=%d, free slot found at %d\n", capacity, size, slot);
		data[slot] = element;
		size++;
	}
	
	void add(int index, Object element) {
		data[getFreeSlot(index)] = element;
		size++;
	}
	
	void clear() {
		size = 0;
	}
	
	Object get(int index) {
		
		int slot = indexToSlot(index);
		if(slot == -1) {
			
		}
		return data[slot];
	}
	
	int indexOf(Object element) {
		
		if(element == null) {
			// No nulls are allowed in a sparse list
			return -1;
		}
		
		int counter = 0;
		for(int i: 0..capacity) {
			if(data[i] == element) {
				break;
			} else if(data[i] != null) {
				counter++;
			}
		}
		
		return counter;
		
	}
	
	int lastIndexOf(Object element) {
		
		if(element == null) {
			// No nulls are allowed in a sparse list
			return -1;
		}
		
		int counter = size - 1;
		for(int i = capacity - 1; i >= 0; i--) {
			if(data[i] == element) {
				break;
			} else if(data[i] != null) {
				counter--;
			}
		}
		
		return counter;
		
	}
	
	Object remove(int index) {
		int slot = indexToSlot(index);
		Object element = data[slot];
		data[slot] = null;
		size--;
		return element;
	}
	
	boolean removeElement(Object element) {
		int slot = slotOf(element);
		if(slot != -1) {
			data[slot] = null;
			size--;
			return true;
		} else {
			return false;
		}
	}
	
	void set(int index, Object element) {
		int slot = indexToSlot(index);
		if(slot != -1) {
			data[slot] = element;
		}
	}
	
	/** private */
	int slotOf(Object element) {
		
		for(int i: 0..capacity) {
			if(data[i] == element) {
				return i;
			}
		}
		return -1;
		
	}
	
	/**
	 * private
	 * 
	 * Here is quick rundown of how the function works:
	 *  - There is a distinction between an index and a slot. Indices
	 * are all the user sees. Slots are the real position of data in
	 * the array. Thus, often a slot number is greater than an index
	 * number, as there are holes in the data.
	 *  - The 'minimum' parameter is how much elements must there be before
	 * the free slot returned. If you don't care where the free slot is,
	 * -1 should be passed as a parameter
	 */
	int getFreeSlot(int minimum) {
		
		if(minimum == -1) {
			// If we don't care, start at -1
			// In the future, this may be randomized
			// to allow for *statically* faster inserts
		}
		
		int freeSlot = -1;
		
		// Counts the number of elements before the free slot
		int counter = 0;
		
		for(int i: 0..capacity) {
			if(data[i] == null) {
				// A free slot! Great! But do we have at
				// least 'minimum' elements before us ?
				if(counter >= minimum) {
					// Remember, a slot is the position in the real array
					freeSlot = i;
					break;
				}
			} else {
				if(counter >= minimum && minimum != -1) {
					// Huh oh, we reached the wanted index, and still no
					// free space ! We gotta grow.
					if(data[capacity - 1] != null) {
						grow();
					}
					memmove(data+i+1, data+i, (size-i) * sizeof(Object));
					freeSlot = i;
					break;
				}
				// Stumbled upon an element, increase the counter
				counter++;
			}
		}
		
		// No free slot found ?
		if(freeSlot == -1) {
			// Doesn't matter, we'll find room
			freeSlot = capacity;
			// Now pay the rent.
			grow();
		}
		
		// Now, except if malloc went woopsie-daisy, we have at least one free slot
		return freeSlot;
		
	}

	/**	
	 * private 
	 */
	int indexToSlot(int index) {
		
		int slot = -1;
		int counter = 0;
		for(int i: 0..capacity) {
			if(data[i] != null) {
				if(counter >= index) {
					slot = i;
					break;
				}
				counter++;
			}
		}
		
		return slot;
		
	}

	/** private */	
	void grow() {
		capacity = capacity * 1.1 + 10;
		Object* tmpData = realloc(data, capacity * sizeof(Object));
		if(tmpData != null) {
			data = tmpData;
		} else {
			printf("Failed to allocate %d bytes of memory for array to grow! Exiting..\n", capacity * sizeof(Object));
			exit(1);
		}
	}
	
	/** private */
	boolean isValidIndex(int i) {
		if(i < 0) {
			printf("ArrayList: ArrayIndexOutOfBoundException: index=%d < 0\n", i);
			return false;
		} else if(i >= size) {
			printf("ArrayList: ArrayIndexOutOfBoundException: index=%d >= size=%d\n", i, size);
			return false;
		}
		return true;
	}

}
